import { isFunction, isNil, omit } from 'lodash';
import { PostOrderType } from './../constants';
import { PaginateOptions, QueryHook } from '@/modules/database/types';
import { PostRepository } from './../repositories/post.repository';
import { Injectable } from "@nestjs/common";
import { PostEntity } from '../entities/post.entity';
import { EntityNotFoundError, IsNull, Not, SelectQueryBuilder } from 'typeorm';
import { paginate } from '@/modules/database/helpers';

@Injectable()
export class PostService {
    constructor(protected repository: PostRepository) { }
    /**
     * 获取分页数据
     * @param options 分页选项
     * @param callback 添加额外的查询
     */
    async paginate(options: PaginateOptions, callback?: QueryHook<PostEntity>) {
        const qb = await this.buildListQuery(this.repository.buildBaseQB(), options, callback);
        return paginate(qb, options);
    }
    /**
     * 查询单篇文章
     * @param id 文章id
     * @param callback 额外的查询
     */
    async detail(id: string, callback?: QueryHook<PostEntity>) {
        let qb = this.repository.buildBaseQB();
        qb.where(`post.id = :id`, { id });
        qb = !isNil(callback) && isFunction(callback) ? await callback(qb) : qb;
        const item = await qb.getOne();
        if (!item) {
            throw new EntityNotFoundError(PostEntity, `Post with id ${id} not found`);
        }
        return item;
    }
    /**
     * 创建文章
     * @param data 文章
     */
    async create(data: Record<string, any>) {
        const item = await this.repository.save(data);
        return this.detail(item.id)
    }
    /**
     * 更新文章
     * @param data 
     */
    async update(data: Record<string, any>) {
        await this.repository.update(data.id, omit(data, ['id']));
        return this.detail(data.id);
    }
    /**
     * 删除文章
     * @param id 
     */
    async delete(id: string) {
        const item = await this.repository.findOneByOrFail({ id });
        return this.repository.remove(item);
    }

    /**
     * 构建文章列表查询器
     * @param qb 初始查询构造器
     * @param options 排查分页选项后的查询选项
     * @param callback 添加额外的查询
     */
    buildListQuery(qb: SelectQueryBuilder<PostEntity>, options: Record<string, any>, callback: QueryHook<PostEntity>) {
        const { orderBy, isPublished } = options;
        if (typeof isPublished === 'boolean') {
            isPublished ? qb.where({ publishedAt: Not(IsNull()) }) : qb.where({ publishedAt: IsNull() });
        }

        this.queryOrderBy(qb, orderBy);
        if (callback) return callback(qb);
        return qb;
    }
    /**
     * 对文章进行排序的query构建
     * @param qb 查询构造器
     * @param orderBy 排序方式
     */
    queryOrderBy(qb: SelectQueryBuilder<PostEntity>, orderBy?: PostOrderType) {
        switch (orderBy) {
            case PostOrderType.CREATED:
                qb.orderBy('createdAt', 'DESC');
                break;
            case PostOrderType.UPDATED:
                qb.orderBy('updatedAt', 'DESC');
                break;
            case PostOrderType.PUBLISHED:
                qb.orderBy('publishedAt', 'DESC');
                break;
            case PostOrderType.CUSTOM:
                qb.orderBy('customOrder', 'DESC');
                break;
            default:
                qb.orderBy('createdAt', 'DESC');
                qb.addOrderBy('updatedAt', 'DESC');
                qb.addOrderBy('publishedAt', 'DESC');
                break;
        }
    }
}